//%module rumd // rumd is the default module name, no need to explicitly say so



%init %{
  // Initialize the GPU device as part of module initialization
  int argc = 0;
  char* argv[] = {};
  try {
    Device::GetDevice().Init(argc, argv);
  }
  catch (RUMD_Error e){
    std::cerr << "RUMD_Error thrown from function " << e.className << "::" << e.methodName << ": " << std::endl << e.errorStr << std::endl;
    throw;
  }
  // Initialize the numpy module so we can use its C API (otherwise get seg 
  // faults)
  import_array();
  %}

%{
#include "rumd/Device.h" // curiously, leaving this out generates not a compile-error but a linker error....
#include "numpy/arrayobject.h"
%}



// the following allows exceptions to be passed to Python
%include exception.i
%{
#include "rumd/RUMD_Error.h"
  %}

%exception {
  try {
    $action
      } catch (RUMD_Error e) {
    std::string errorStr("In class ");
    errorStr += e.className;
    errorStr += ", method ";
    errorStr += e.methodName;
    errorStr += ": ";
    errorStr += e.errorStr;
    SWIG_exception(SWIG_RuntimeError,errorStr.c_str()); 
  } catch(...) {
    SWIG_exception(SWIG_RuntimeError, "Unknown error");
  }
 }

%feature("kwargs"); // enable keyword arguments


%include "stringTypeMap.i"

%include "SimulationBox.i"

%include "Sample.i"

%include "Potentials.i"

%include "Integrators.i"

%include "IO.i"

%include "ExternalCalculators.i"

%include "MoleculeData.i"
