/*
    Copyright (C) 2010  Thomas Schrøder

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    LICENSE.txt file for license details.
*/

#include "rumd/EnergiesMetaData.h"
#include "rumd/RUMD_Error.h"
#include <cstring>
#include <cassert>
#include <cerrno>
#include <cmath>
#include <zlib.h>
#include <vector>

class rumd_stats {
public:  
  rumd_stats();
  ~rumd_stats() { Free(); }
  void SetVerbose(bool v) { verbose = v; }
  void SetDirectory(const std::string& directory) {this->directory = directory;}

  void ComputeStats(unsigned int first_block=0, int last_block=-1);
  void PrintStats();
  void WriteStats();

  unsigned int GetNumCols() const { return num_cols; }
  unsigned int GetCount() const { return count; }

  const double* GetMeanVals() const {
    if(!mean_vals) throw RUMD_Error("rumd_stats","GetMeanVals","mean_vals has not been computed yet");
    return mean_vals; 
  }

  const double* GetMeanSqVals() const { 
    if(!meanSq_vals) throw RUMD_Error("rumd_stats","GetMeanSqVals","meanSq_vals has not been computed yet");
    return meanSq_vals;
  }
  
  const double* GetCovarianceVals() const {
    if(!covariance_vals) throw RUMD_Error("rumd_stats","GetCovarianceVals","covariance_vals has not been computed yet");
    return covariance_vals;
  }

  const double* GetDriftVals() const {
    if(!drift_vals) throw RUMD_Error("rumd_stats","GetDriftVals","drift_vals has not been computed yet");
    return drift_vals;
  }

  std::string GetColumnLabel(unsigned int colIdx) {
    if(colIdx >= num_cols)
      throw RUMD_Error("rumd_stats","GetColumnLabel","Index is too large, or yo u haven't called ComputeStats yet.");
    return column_labels[colIdx];
}


private:
  void Allocate(unsigned int set_num_cols);
  void Free();

  rumd_stats(const rumd_stats&);
  rumd_stats& operator=(const rumd_stats&);
  std::string directory;
  std::vector<std::string> column_labels;
  bool verbose;
  double* mean_vals;
  double* meanSq_vals;
  double* covariance_vals;
  double* drift_vals;
  unsigned int num_cols;
  unsigned int count;
  EnergiesMetaData EMD;
  unsigned int numFilenameDigits;
};
