
#ifndef ANGLEPOTENTIAL_H
#define ANGLEPOTENTIAL_H

#include "rumd/Potential.h"

class MoleculeData; class Sample;

///////////////////////////////////////////////////////
// Abstract BondPotential class
///////////////////////////////////////////////////////

class AnglePotential : public Potential{

 public:
  MoleculeData *M;
  Sample *S;

  int num_blocks;
  int num_threads;

};



// Cosine squared potential

class AngleCosSq : public AnglePotential{
 private:
  AngleCosSq(const AngleCosSq&);
  AngleCosSq& operator=(const AngleCosSq&);
  
 public:
  AngleCosSq(MoleculeData *, Sample *);
  void CalcF(bool initialize, bool calc_stresses);
  double GetPotentialEnergy();
};


///////////////////////////////////////////////////////
// Kernel prototypes
///////////////////////////////////////////////////////


template <int energy, class Simbox> 
__global__ void CosSq( float4 *r, float4 *f,  Simbox *simbox, float *simboxpointer, 
		       uint4 *alist, float2 *parameter, float *d_epot_angle, 
		       float *d_angles, int num_thread_per_block, int num_angles );


//////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////
__global__ void AngleResetForces( float4 *f, float4 *w );
__device__ inline void AngleAtomicFloatAdd(float *address, float val);

#endif
