
#ifndef BONDPOTENTIAL_H
#define BONDPOTENTIAL_H

#include "rumd/Potential.h"

class MoleculeData; class Sample;

///////////////////////////////////////////////////////
// Abstract BondPotential class
///////////////////////////////////////////////////////

class BondPotential : public Potential{
 public:
  MoleculeData *M;
  Sample *S;
};

///////////////////////////////////////////////////////
// Specialized bond potentials
///////////////////////////////////////////////////////

// Harmonic
class BondHarmonic : public BondPotential{
 private:
  BondHarmonic(const BondHarmonic&);
  BondHarmonic& operator=(const BondHarmonic&);

 public:
  BondHarmonic(MoleculeData *, Sample *);
  void CalcF(bool initialize, bool calc_stresses);
  double GetPotentialEnergy();
};

// FENE
class BondFENE : public BondPotential{
 private:
  BondFENE(const BondFENE&);
  BondFENE& operator=(const BondFENE&);
  
 public:
  BondFENE(MoleculeData *, Sample *);
  void CalcF(bool initialize, bool calc_stresses);
  double GetPotentialEnergy();
};


///////////////////////////////////////////////////////
// Kernel prototypes
///////////////////////////////////////////////////////

template <int stress, int energy, bool initialize, class Simbox> 
  __global__ void Harmonic( uint1 *blist, float4 *r, float4 *f, float4 *w, 
			    unsigned tpb, unsigned max_num_bonds, unsigned npart, 
			    unsigned virt_part, Simbox *simbox, float *simboxpointer, 
			    int1 *btlist, float2 *bplist, int1 *btlist_int );

template <int stress, int energy, bool initialize, class Simbox> 
  __global__ void FENE( uint1 *blist, float4 *r, float4 *f, float4 *w,  
			unsigned tpb, unsigned max_num_bonds, unsigned npart, 
			unsigned virt_part, Simbox *simbox, float *simboxpointer, 
			int1 *btlist, int1 *btlist_int );

#endif // BONDPOTENTIAL_H
