#ifndef NEIGHBORLIST_H
#define NEIGHBORLIST_H

/*
    Copyright (C) 2010  Thomas Schrøder

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    LICENSE.txt file for license details.
*/

#include "rumd/rumd_technical.h"
#include "rumd/rumd_base.h"
#include "rumd/ParticleData.h"
#include "rumd/KernelPlan.h"

class PairPotential; class SimulationBox;

class NeighborList{
 
 private:
  NeighborList(const NeighborList&); 
  NeighborList& operator=(const NeighborList& N);

  ParticleData* particleData;
  KernelPlan kp;

  NBTYPEv* h_ExclusionList;
  NBTYPEv* d_ExclusionList;
  NBTYPEv* d_savedNeighborList;

  // Dynamic allocation
  unsigned int numBlocks;
  unsigned int numBlocksParticles;
  unsigned int numParticles;
  unsigned int particlePerBlock;
  unsigned int numVirtualParticles; 

  int* d_ctrl;
  int* cell_list;

  float h_shell[MaxNumTypes*MaxNumTypes];
  float* d_shell;

  float skin;
  float4* d_last_r;
  
  void Free();
  
 public:
  NeighborList();
  ~NeighborList();
  
  NBTYPEv* NbList; // This should be made private.
  
  void Initialize( ParticleData* pData, KernelPlan& kPlan);
  void UpdateNBlist(SimulationBox* simBox);
  void SaveNeighborList();
  void RestoreNeighborList();
  void ResetNeighborList();
  
  void PrintExclusionList();
  void PrintCellList();

  // Set methods.
  void SetCutoffs(PairPotential* potential);
  void SetCutoffs(PairPotential* potential, float skin);
  void SetExclusionList( NBTYPEv* newList );

  void SetExclusionType( unsigned* position, unsigned type0, unsigned type1 );

  void SetExclusionBond(int1 *h_btlist, uint1 *h_blist, unsigned num_part, unsigned num_virt_part, 
			unsigned max_num_bonds, unsigned etype); 
  void SetExclusionAngle(uint4 *h_alist, unsigned num_angles);
  void SetExclusionDihedral(uint4 *h_dlist, unsigned num_dihedrals);
  void SetExclusionMolecule(int1 *h_mlist, unsigned molindex, unsigned max_num_uau);
  
  // Get methods.
  float GetSkin() const { return skin; }
};

__global__ void md_initialize_nblist_check( int* updateNbList );

template <class S> __global__ void displacement_since_nb_build( unsigned numParticles, float4* d_r, float4* d_last_r, float4* d_im,
								S* simBox, float* simBoxPointer, float skin2over_4, int* updateNbList);

template <class S> __global__ void md_update_nblist( unsigned int num_part, float4* r, float4* im, float* params, 
						     S* simBox, float* simBoxPointer, NBTYPEv* NbList, NBTYPEv* ExclusionList, 
						     int* cell_list, int *updateNbList, float4* last_r );

#endif // NEIGHBORLIST_H
